﻿using System;
using System.Data;
using System.Text;
using System.Threading;
using System.Collections;

using Xoa.Virtual.Gangs;
using Xoa.Managers;
using Xoa.Virtual.Rooms;
using Microsoft.VisualBasic;
using Ion.Storage;

namespace Xoa.Virtual.Rooms.Bots
{
    /// <summary>
    /// Represents a virtual user in a virtual room.
    /// </summary>
    public class virtualRoomBot
    {
        /// <summary>
        /// The database ID of the user.
        /// </summary>
        internal int botID;
        /// <summary>
        /// The ID of the virtual room where the user is in.
        /// </summary>
        internal int roomID;
        /// <summary>
        /// The ID of the user in the virtual room.
        /// </summary>
        internal int roomUID;
        /// <summary>
        /// The X position of the user in the virtual room.
        /// </summary>
        internal int X;
        /// <summary>
        /// The Y position of the user in the virtual room.
        /// </summary>
        internal int Y;
        /// <summary>
        /// The height of the user in the virtual room.
        /// </summary>
        internal double H;
        /// <summary>
        /// The rotation of the user's head in the virtual room.
        /// </summary>
        internal byte Z1;
        /// <summary>
        /// The rotation of the user's body in the virtual room.
        /// </summary>
        internal byte Z2;
        /// <summary>
        /// The X position where the user wants to walk to. If not walking, -1.
        /// </summary>
        internal int goalX;
        /// <summary>
        /// The Y position where the user wants to walk to. If not walking, -1.
        /// </summary>
        internal int goalY;
        /// <summary>
        /// Specifies if the user can walk.
        /// </summary>
        internal bool walkLock;
        /// <summary>
        /// Specifies if the user is on his way of leaving the room after clicking the door.
        /// </summary>
        internal bool walkDoor;
        /// <summary>
        /// Guestroom only. Specifies if the user has voted on the guestroom already.
        /// </summary>
        internal bool hasVoted;
        /// The room user ID (rUID) of the virtual user where this virtual user is currently trading items with. If not trading, then this value is -1.
        /// </summary>
        internal int _tradePartnerRoomUID = -1;
        /// <summary>
        /// Swimming pools only. Specifies the swimming pool outfit.
        /// </summary>
        internal string swimOutfit;
        /// <summary>
        /// Gamelobbies only. Specifies the amount of gamepoints that this user has for the game in the current gamelobby.
        /// </summary>
        internal int gamePoints;
        /// <summary>
        /// The parent virtual user of this room user.
        /// </summary>
        internal virtualBot Bot;
        /// <summary>
        /// The status manager of this room user.
        /// </summary>
        internal virtualRoomBotStatusManager statusManager;
        /// <summary>
        /// Indicates if the room user is currently typing a chat message.
        /// </summary>
        internal bool isTyping;
        /// <summary>
        /// Indicates if the room user is able to use the special teleport cast, which allows him to teleport around the room.
        /// </summary>
        internal bool SPECIAL_TELEPORTABLE = false;
        private delegate void walkSleepPointer();
        /// <summary>
        /// Initializes a virtual room user.
        /// </summary>
        /// <param name="userID">The database ID of the virtual user of this room user object.</param>
        /// <param name="roomID">The database ID of the room where the room user is in.</param>
        /// <param name="roomUID">The ID that identifies the room user in the virtual room.</param>
        /// <param name="User">The parent vittual user of this room user.</param>
        /// <param name="statusManager">The status manager of this room user.</param>
        internal virtualRoomBot(int botID, int roomID, int roomUID, virtualBot Bot, virtualRoomBotStatusManager statusManager)
        {
            this.botID = botID;
            this.roomID = roomID;
            this.roomUID = roomUID;
            this.Bot = Bot;
            this.statusManager = statusManager;
        }
        /// <summary>
        /// The virtualRoom object that represents the room where this room user is in.
        /// </summary>
        internal virtualRoom Room
        {
            get
            {
                return roomManager.getRoom(roomID);
            }
        }
        /// <summary>
        /// Refreshes the status of this room user in the virtual room.
        /// </summary>
        internal void Refresh()
        {
            Room.Refresh(this);
        }
        /// <summary>
        /// Returns the details string for this room user, containing username etc.
        /// </summary>
        internal string detailsString
        {
            get
            {
                using (DatabaseClient dbClient = Xoa.Eucalypt.dbManager.GetClient())
                {
                    if (Bot.Animal != "-1")
                    {
                        // old pet string
                        //string s = "i:" + roomUID + Convert.ToChar(13) + "n:" + roomUID + Convert.ToChar(4) + Bot.Name + Convert.ToChar(13) + "f:" + Bot.Animal + " " + Bot.Animal + " " + Bot.Colour + Convert.ToChar(13) + "l:" + X + " " + Y + " " + H + Convert.ToChar(13);
                        //s += "c:" + Convert.ToChar(13);
                        //return s;
                        string Figure = Bot.Animal + " " + Bot.Animal + " " + Bot.Colour + "";
                        string s = "i:" + roomUID + Convert.ToChar(13) + "n:" + roomUID + Convert.ToChar(4) + Bot.Name + Convert.ToChar(13) + "f:" + Figure + Convert.ToChar(13) + "l:" + X + " " + Y + " " + H + Convert.ToChar(13);
                        s += "c:" + Convert.ToChar(13);
                        return s;
                    }
                    else
                    {
                        string s = "i:" + roomUID + Convert.ToChar(13) + "a:-1" + Convert.ToChar(13) + "n:" + Bot.Name + Convert.ToChar(13) + "f:" + Bot.Figure + Convert.ToChar(13) + "l:" + X + " " + Y + " " + H + Convert.ToChar(13);
                        if (Bot.Mission != "")
                        {
                            s += "c:" + Bot.Mission + Convert.ToChar(13);
                        }
                        s += "b:" + "1:NWB" + Convert.ToChar(13);

                        return s;
                    }
                }
            }
        }
        /// <summary>
        /// Returns the action string for this room user, containing position, movements, statuses etc.
        /// </summary>
        internal string statusString
        {
            get
            {
                string s = roomUID + " " + X + "," + Y + "," + H.ToString().Replace(",", ".") + "," + Z1 + "," + Z2 + "/";
                foreach (string Key in statusManager._Statuses.Keys)
                {
                    s += Key;
                    string Value = (string)statusManager._Statuses[Key];
                    if (Value != "")
                        s += " " + Value;
                    s += "/";
                }
                return s;
            }
        }
    }
}
